﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Clients.Common;
using HeuristicLab.Collections;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Clients.OKB {
  [Item("OKB", "OKB front-end.")]
  public sealed class OKB : IContent {
    private static OKB instance;
    public static OKB Instance {
      get {
        if (instance == null) instance = new OKB();
        return instance;
      }
    }

    #region Properties
    private ItemCollection<Platform> platforms;
    public ItemCollection<Platform> Platforms {
      get { return platforms; }
    }
    private ItemCollection<DataType> dataTypes;
    public ItemCollection<DataType> DataTypes {
      get { return dataTypes; }
    }
    private IEnumerable<User> users;
    public IEnumerable<User> Users {
      get { return users; }
    }
    private ItemCollection<AlgorithmClass> algorithmClasses;
    public ItemCollection<AlgorithmClass> AlgorithmClasses {
      get { return algorithmClasses; }
    }
    private ItemCollection<Algorithm> algorithms;
    public ItemCollection<Algorithm> Algorithms {
      get { return algorithms; }
    }
    private ItemCollection<ProblemClass> problemClasses;
    public ItemCollection<ProblemClass> ProblemClasses {
      get { return problemClasses; }
    }
    private ItemCollection<Problem> problems;
    public ItemCollection<Problem> Problems {
      get { return problems; }
    }
    #endregion

    private OKB() { }

    #region Refresh
    public void Refresh() {
      OnRefreshing();
      if (platforms == null) {
        platforms = new ItemCollection<Platform>();
        platforms.ItemsRemoved += new CollectionItemsChangedEventHandler<Platform>(platforms_ItemsRemoved);
      }
      platforms.Clear();
      if (dataTypes == null) {
        dataTypes = new ItemCollection<DataType>();
        dataTypes.ItemsRemoved += new CollectionItemsChangedEventHandler<DataType>(dataTypes_ItemsRemoved);
      }
      dataTypes.Clear();
      if (algorithmClasses == null) {
        algorithmClasses = new ItemCollection<AlgorithmClass>();
        algorithmClasses.ItemsRemoved += new CollectionItemsChangedEventHandler<AlgorithmClass>(algorithmClasses_ItemsRemoved);
      }
      algorithmClasses.Clear();
      if (algorithms == null) {
        algorithms = new ItemCollection<Algorithm>();
        algorithms.ItemsRemoved += new CollectionItemsChangedEventHandler<Algorithm>(algorithms_ItemsRemoved);
      }
      algorithms.Clear();
      if (problemClasses == null) {
        problemClasses = new ItemCollection<ProblemClass>();
        problemClasses.ItemsRemoved += new CollectionItemsChangedEventHandler<ProblemClass>(problemClasses_ItemsRemoved);
      }
      problemClasses.Clear();
      if (problems == null) {
        problems = new ItemCollection<Problem>();
        problems.ItemsRemoved += new CollectionItemsChangedEventHandler<Problem>(problems_ItemsRemoved);
      }
      problems.Clear();

      var call = new Func<Exception>(delegate() {
        try {
          platforms.AddRange(CallAdminService<Platform[]>(s => s.GetPlatforms()).OrderBy(x => x.Name));
          dataTypes.AddRange(CallAdminService<DataType[]>(s => s.GetDataTypes()).OrderBy(x => x.Name));
          users = CallAuthenticationService<User[]>(s => s.GetUsers()).OrderBy(x => x.Name);
          algorithmClasses.AddRange(CallAdminService<AlgorithmClass[]>(s => s.GetAlgorithmClasses()).OrderBy(x => x.Name));
          algorithms.AddRange(CallAdminService<Algorithm[]>(s => s.GetAlgorithms()).OrderBy(x => x.Name));
          problemClasses.AddRange(CallAdminService<ProblemClass[]>(s => s.GetProblemClasses()).OrderBy(x => x.Name));
          problems.AddRange(CallAdminService<Problem[]>(s => s.GetProblems()).OrderBy(x => x.Name));
          return null;
        }
        catch (Exception ex) {
          return ex;
        }
      });
      call.BeginInvoke(delegate(IAsyncResult result) {
        Exception ex = call.EndInvoke(result);
        if (ex != null) ErrorHandling.ShowErrorDialog("Refresh failed.", ex);
        OnRefreshed();
      }, null);
    }
    #endregion

    #region Store
    public bool Store(IOKBItem item) {
      try {
        if (item.Id == 0) {
          if (item is Platform)
            item.Id = CallAdminService<long>(s => s.AddPlatform((Platform)item));
          else if (item is DataType)
            item.Id = CallAdminService<long>(s => s.AddDataType((DataType)item));
          else if (item is AlgorithmClass)
            item.Id = CallAdminService<long>(s => s.AddAlgorithmClass((AlgorithmClass)item));
          else if (item is Algorithm)
            item.Id = CallAdminService<long>(s => s.AddAlgorithm((Algorithm)item));
          else if (item is ProblemClass)
            item.Id = CallAdminService<long>(s => s.AddProblemClass((ProblemClass)item));
          else if (item is Problem)
            item.Id = CallAdminService<long>(s => s.AddProblem((Problem)item));
        } else {
          if (item is Platform)
            CallAdminService(s => s.UpdatePlatform((Platform)item));
          else if (item is DataType)
            CallAdminService(s => s.UpdateDataType((DataType)item));
          else if (item is AlgorithmClass)
            CallAdminService(s => s.UpdateAlgorithmClass((AlgorithmClass)item));
          else if (item is Algorithm)
            CallAdminService(s => s.UpdateAlgorithm((Algorithm)item));
          else if (item is ProblemClass)
            CallAdminService(s => s.UpdateProblemClass((ProblemClass)item));
          else if (item is Problem)
            CallAdminService(s => s.UpdateProblem((Problem)item));
        }
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Store failed.", ex);
        return false;
      }
    }
    #endregion

    #region Algorithm Methods
    public Guid[] GetAlgorithmUsers(long algorithmId) {
      try {
        return CallAdminService<Guid[]>(s => s.GetAlgorithmUsers(algorithmId));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Refresh authorized algorithm users failed.", ex);
        return null;
      }
    }
    public bool UpdateAlgorithmUsers(long algorithmId, Guid[] users) {
      try {
        CallAdminService(s => s.UpdateAlgorithmUsers(algorithmId, users));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Update authorized algorithm users failed.", ex);
        return false;
      }
    }
    public AlgorithmData GetAlgorithmData(long algorithmId) {
      try {
        return CallAdminService<AlgorithmData>(s => s.GetAlgorithmData(algorithmId));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Refresh algorithm data failed.", ex);
        return null;
      }
    }
    public bool UpdateAlgorithmData(AlgorithmData algorithmData) {
      try {
        CallAdminService(s => s.UpdateAlgorithmData(algorithmData));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Update algorithm data failed.", ex);
        return false;
      }
    }
    #endregion

    #region Problem Methods
    public Guid[] GetProblemUsers(long problemId) {
      try {
        return CallAdminService<Guid[]>(s => s.GetProblemUsers(problemId));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Refresh authorized problem users failed.", ex);
        return null;
      }
    }
    public bool UpdateProblemUsers(long problemId, Guid[] users) {
      try {
        CallAdminService(s => s.UpdateProblemUsers(problemId, users));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Update authorized problem users failed.", ex);
        return false;
      }
    }
    public ProblemData GetProblemData(long problemId) {
      try {
        return CallAdminService<ProblemData>(s => s.GetProblemData(problemId));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Refresh problem data failed.", ex);
        return null;
      }
    }
    public bool UpdateProblemData(ProblemData problemData) {
      try {
        CallAdminService(s => s.UpdateProblemData(problemData));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Update problem data failed.", ex);
        return false;
      }
    }
    #endregion

    #region Events
    public event EventHandler Refreshing;
    private void OnRefreshing() {
      EventHandler handler = Refreshing;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    public event EventHandler Refreshed;
    private void OnRefreshed() {
      EventHandler handler = Refreshed;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    private void platforms_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Platform> e) {
      try {
        foreach (Platform p in e.Items)
          CallAdminService(s => s.DeletePlatform(p.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void dataTypes_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<DataType> e) {
      try {
        foreach (DataType d in e.Items)
          CallAdminService(s => s.DeleteDataType(d.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void algorithmClasses_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<AlgorithmClass> e) {
      try {
        foreach (AlgorithmClass a in e.Items)
          CallAdminService(s => s.DeleteAlgorithmClass(a.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void algorithms_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Algorithm> e) {
      try {
        foreach (Algorithm a in e.Items)
          CallAdminService(s => s.DeleteAlgorithm(a.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void problemClasses_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<ProblemClass> e) {
      try {
        foreach (ProblemClass p in e.Items)
          CallAdminService(s => s.DeleteProblemClass(p.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void problems_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Problem> e) {
      try {
        foreach (Problem p in e.Items)
          CallAdminService(s => s.DeleteProblem(p.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    #endregion

    #region Helpers
    private void CallAdminService(Action<IOKBService> call) {
      OKBServiceClient client = ClientFactory.CreateClient<OKBServiceClient, IOKBService>();
      try {
        call(client);
      }
      finally {
        try {
          client.Close();
        }
        catch (Exception) {
          client.Abort();
        }
      }
    }
    private T CallAdminService<T>(Func<IOKBService, T> call) {
      OKBServiceClient client = ClientFactory.CreateClient<OKBServiceClient, IOKBService>();
      try {
        return call(client);
      }
      finally {
        try {
          client.Close();
        }
        catch (Exception) {
          client.Abort();
        }
      }
    }
    private T CallAuthenticationService<T>(Func<IAuthenticationService, T> call) {
      AuthenticationServiceClient client = ClientFactory.CreateClient<AuthenticationServiceClient, IAuthenticationService>();
      try {
        return call(client);
      }
      finally {
        try {
          client.Close();
        }
        catch (Exception) {
          client.Abort();
        }
      }
    }
    #endregion
  }
}
